/*
 * Copyright (c) 2004-2016 Zementis, Inc. All rights reserved.
 */
package com.zementis.adapa.sample.repository;

import static com.mongodb.client.model.Filters.eq;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.Serializable;
import java.nio.charset.Charset;
import java.util.HashSet;
import java.util.Set;

import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.SerializationUtils;
import org.bson.Document;
import org.bson.types.Binary;

import com.mongodb.MongoClient;
import com.mongodb.client.MongoCollection;
import com.mongodb.client.MongoCursor;
import com.mongodb.client.MongoDatabase;
import com.mongodb.client.model.Filters;
import com.zementis.adapa.extensions.AssetRepository;

/**
 * A sample {@link AssetRepository} for MongoDB.
 * 
 */
public class MongoAssetRepository implements AssetRepository {

    private static final String ADAPA_DB = "adapa";

    private static final String ASSET_COLLECTION = "asset";

    private static final String ASSET_SOURCE = "source";

    private static final String ASSET_ANNOTATED_SOURCE = "annotated_source";

    private static final String ASSET_BYTE_STREAM = "byte_stream";

    private static final String ASSET_INFO = "info";

    private static final String IDENTIFIER = "identifier";

    private static final String ACTIVE = "active";

    private final MongoDatabase database;

    @SuppressWarnings("resource")
    public MongoAssetRepository(String hostname, int port) {
        MongoClient mongoClient;
        try {
            mongoClient = new MongoClient(hostname, port);
        } catch (Exception ex) {
            throw new RuntimeException("Could not create MongoClient");
        }
        database = mongoClient.getDatabase(ADAPA_DB);
    }

    public String addAsset(Serializable adapaAssetInfo, InputStream adapaAssetSource) {
        return addAsset(adapaAssetInfo, adapaAssetSource, null);
    }

    @Override
    public String addAsset(Serializable adapaAssetInfo, InputStream adapaAssetSource,
        InputStream annotatedAdapaAssetSource) {
        return addAsset(adapaAssetInfo, adapaAssetSource, annotatedAdapaAssetSource, null);
    }

    @Override
    public String addAsset(Serializable adapaAssetInfo, InputStream adapaAssetSource,
        InputStream annotatedAdapaAssetSource, InputStream byteStream) {
        MongoCollection<Document> assets = database.getCollection(ASSET_COLLECTION);
        Document document = new Document();
        String identifier = String.valueOf(System.currentTimeMillis());
        document.put(IDENTIFIER, identifier);
        document.put(ACTIVE, true);
        try {
            document.put(ASSET_INFO, SerializationUtils.serialize(adapaAssetInfo));
            String source = IOUtils.toString(adapaAssetSource, Charset.defaultCharset());
            document.put(ASSET_SOURCE, source);
            if (annotatedAdapaAssetSource != null) {
                String annotatedSource =
                    IOUtils.toString(annotatedAdapaAssetSource, Charset.defaultCharset());
                document.put(ASSET_ANNOTATED_SOURCE, annotatedSource);
            }
            if (byteStream != null) {
                document.put(ASSET_BYTE_STREAM, IOUtils.toByteArray(byteStream));
            }
            assets.insertOne(document);
            return identifier;
        } catch (Exception ex) {
            throw new RuntimeException("Unable to insert asset into database", ex);
        }
    }

    public Set<String> getAssetIds() {
        MongoCollection<Document> assets = database.getCollection(ASSET_COLLECTION);
        Set<String> assetIds = new HashSet<String>();
        MongoCursor<Document> cursor = assets.find().iterator();
        while (cursor.hasNext()) {
            assetIds.add(cursor.next().get(IDENTIFIER).toString());
        }
        return assetIds;
    }

    @Override
    public Set<String> getActiveAssetIds() {
        MongoCollection<Document> assets = database.getCollection(ASSET_COLLECTION);
        Set<String> assetIds = new HashSet<String>();
        MongoCursor<Document> cursor = assets.find(eq(ACTIVE, true)).iterator();
        while (cursor.hasNext()) {
            assetIds.add(cursor.next().get(IDENTIFIER).toString());
        }
        return assetIds;
    }

    public Serializable getAssetInfo(String identifier) {
        MongoCollection<Document> assets = database.getCollection(ASSET_COLLECTION);
        Document doc = assets.find(Filters.eq(IDENTIFIER, identifier)).first();
        if (doc != null) {
            Binary adapaAssetBytes = (Binary) doc.get(ASSET_INFO);
            return (Serializable) SerializationUtils.deserialize(adapaAssetBytes.getData());
        }
        return null;
    }

    public InputStream getAssetSource(String identifier) {
        MongoCollection<Document> assets = database.getCollection(ASSET_COLLECTION);
        Document doc = assets.find(Filters.eq(IDENTIFIER, identifier)).first();
        if (doc != null) {
            return IOUtils.toInputStream(doc.getString(ASSET_SOURCE), Charset.defaultCharset());
        }
        return null;
    }

    @Override
    public InputStream getAnnotatedAssetSource(String identifier) {
        MongoCollection<Document> assets = database.getCollection(ASSET_COLLECTION);
        Document doc = assets.find(eq(IDENTIFIER, identifier)).first();
        if (doc != null) {
            return IOUtils.toInputStream(doc.getString(ASSET_ANNOTATED_SOURCE), Charset.defaultCharset());
        }
        return null;
    }

    @Override
    public InputStream getByteStream(String identifier) {
        MongoCollection<Document> assets = database.getCollection(ASSET_COLLECTION);
        Document doc = assets.find(Filters.eq(IDENTIFIER, identifier)).first();
        if (doc != null) {
            Binary byteStream = (Binary) doc.get(ASSET_BYTE_STREAM);
            return new ByteArrayInputStream(byteStream.getData());
        }
        return null;
    }

    @Override
    public void setActive(String identifier, boolean active) {
        MongoCollection<Document> assets = database.getCollection(ASSET_COLLECTION);
        assets.updateOne(eq(IDENTIFIER, identifier), new Document("$set", new Document(ACTIVE, active)));
    }

    @Override
    public void removeAsset(String identifier) {
        MongoCollection<Document> assets = database.getCollection(ASSET_COLLECTION);
        assets.deleteOne(eq(IDENTIFIER, identifier));
    }

}
